﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Datum & Zeit                                                                        //
//  _____________________________________________________                               //
//                                                                                      //
//  Copyright © 2007 Reimar Grasbon                                                     //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "TypeDefs.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Definitionen  ////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
#define     TIME(hour, minute, second)  { hour, minute, second }
#define     TIME_INVALID                TIME( 0xFF , 0xFF, 0xFF)
#define     DATE(year, month, day)      { year, month, day }
#define     DATETIME(date, time)        { date, time }
#define     DATE_INVALID                DATE(0xFFFF, 0xFF, 0xFF)
#define     DATETIME_INVALID            DATETIME(DATE_INVALID, TIME_INVALID)
#define     MIDNIGHT                    TIME(0, 0, 0)
#define     NEW_YEAR_2000               DATE(2000, 1, 1)
#define     SILVESTER_2000              DATETIME( NEW_YEAR_2000, MIDNIGHT)

//////////////////////////////////////////////////////////////////////////////////////////
//  Typen  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
typedef struct               // Zeit:
{   UInt08  hour;            // · Stunde:  0 .. 23
    UInt08  minute;          // · Minute:  0 .. 59
    UInt08  second;          // · Sekunde: 0 .. 59
} Time;

typedef struct               // Datum:
{   UInt16  year;            // · Jahr:   1700 .. 9000
    UInt08  month;           // · Monat:     1 ..   12
    UInt08  day;             // · Monatstag: 1 ..   31
} Date;

typedef struct               // Datum&Zeit:
{   Date    date;            // · Datum
    Time    time;            // · Zeit
} DateTime;

typedef enum                 // Wochentage:
{   DT_NOD   = 0,            // · Kein Wochentag
    DT_MON   = 1,            // · Montag
    DT_TUE   = 2,            // · Dienstag
    DT_WED   = 3,            // · Mittwoch
    DT_THU   = 4,            // · Donnerstag
    DT_FRI   = 5,            // · Freitag
    DT_SAT   = 6,            // · Samstag
    DT_SUN   = 7             // · Sonntag
} DTWeekday;

typedef enum                 // Tagesarten:
{   DT_NORMAL_WORKDAY = 0,   // · Normaler Werktag
    DT_NORMAL_WEEKEND,       // · Normales Wochenende (Samstag/Sonntag)
    DT_NEW_YEAR,             // · Neujahr
    DT_EPIPHANY,             // · Heilig-Dreikönige
    DT_THURSDAY_BEFORE_LENT, // · Weiberfastnacht
    DT_MONDAY_BEFORE_LENT,   // · Rosenmontag
    DT_TUESDAY_BEFORE_LENT,  // · Fastnachtsdienstag
    DT_ASH_WEDNESDAY,        // · Aschermittwoch
    DT_WOMEMS_DAY,           // · Frauentag
    DT_MAUNDY_THURSDAY,      // · Gründonnerstag
    DT_GOOD_FRIDAY,          // · Karfreitag
    DT_EASTER_SUNDAY,        // · Ostersonntag
    DT_EASTER_MONDAY,        // · Ostermontag
    DT_MAY_DAY,              // · Tag der Arbeit
    DT_ASCENSION_THURSDAY,   // · Christi Himmelfahrt
    DT_WHITSUNDAY,           // · Pfingstsonntag
    DT_WHITMONDAY,           // · Pfingstmontag
    DT_CORPUS_CHRISTI,       // · Fronleichnam
    DT_ASSUMPTION_DAY,       // · Mariä Himmelfahrt
    DT_CHILDRENS_DAY,        // · Weltkindertag
    DT_OCTOBER_3RD,          // · Tag der deutschen Einheit
    DT_REFORMATION_DAY,      // · Reformationstag
    DT_ALLHALLOWS,           // · Allerheiligen
    DT_REPENTANCE_DAY,       // · Buß- und Bettag
    DT_NICHOLAS_DAY,         // · Nikolaus
    DT_CHRISTMAS_EVE,        // · Heiligabend
    DT_CHRISTMAS_DAY,        // · 1. Weihnachtstag
    DT_BOXING_DAY,           // · 2. Weihnachtstag
    DT_NEW_YEARS_EVE         // · Silvester
} DTDayType;

typedef enum                 // Zeitphase:
{   DST_MET  = 0,            // · Normalzeit
    DST_MEST = 1,            // · Sommerzeit
    DST_UITS = 2,            // · Zwischenphase im Frühling
    DST_UITA = 3             // · Zwischenphase im Herbst
} DSTPhase;

//////////////////////////////////////////////////////////////////////////////////////////
//  Zeitfunktionen  //////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
bool       TIME_IsValid(const Time*);
void       TIME_ShiftSecond(Time*, SInt08);
void       TIME_ShiftMinute(Time*, SInt08);
void       TIME_ShiftHour(Time*, SInt08);
SInt08     TIME_Compare(const Time*, const Time*);
void       TIME_FromData(const UInt08*, Time*);
void       TIME_ToData(const Time*, UInt08*);

//////////////////////////////////////////////////////////////////////////////////////////
//  Datumsfunktionen  ////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
bool       DATE_IsValid(const Date*);
void       DATE_ShiftDay(Date*, SInt08);
void       DATE_ShiftMonth(Date*, SInt08);
void       DATE_ShiftYear(Date*, SInt08);
SInt08     DATE_Compare(const Date*, const Date*);
bool       DATE_IsLeapYear(UInt16);
UInt08     DATE_DaysInMonth(UInt08, UInt16);
DTWeekday  DATE_WeekDay(const Date*);
SInt16     DATE_YearDay(const Date*);
DTDayType  DATE_DayType(const Date*);
void       DATE_FromData(const UInt08*, Date*);
void       DATE_ToData(const Date*, UInt08*);

//////////////////////////////////////////////////////////////////////////////////////////
//  Datumzeit-Funktionen  ////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
bool       DATETIME_IsValid(const DateTime*);
void       DATETIME_ShiftSecond(DateTime*, SInt08);
void       DATETIME_ShiftMinute(DateTime*, SInt08);
void       DATETIME_ShiftHour(DateTime*, SInt08);
void       DATETIME_ShiftDay(DateTime*, SInt08);
void       DATETIME_ShiftMonth(DateTime*, SInt08);
void       DATETIME_ShiftYear(DateTime*, SInt08);
SInt08     DATETIME_Compare(const DateTime*, const DateTime*);
DSTPhase   DATETIME_DSTPhase(const DateTime*);
void       DATETIME_Localize(DateTime*, SInt08, bool);
void       DATETIME_Universalize(DateTime*, SInt08, bool);
void       DATETIME_FromData(const UInt08*, DateTime*);
void       DATETIME_ToData(const DateTime*, UInt08*);

//////////////////////////////////////////////////////////////////////////////////////////
